# ----------------------------------------------------------------------------
# Copyright (c) 2013--, BP development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

### NOTE: some doctext strings are copied and pasted from manuscript
### http://www.dcc.uchile.cl/~gnavarro/ps/tcs16.2.pdf
import numpy as np

"""root the tree root
levelleftmost(d) / levelrightmost(d) leftmost/rightmost node with depth d
lca(i, j) the lowest common ancestor of two nodes i, j
deepestnode(i) the (first) deepest node in the subtree of i
height(i) the height of i (distance to its deepest node)
degree(i) q = number of children of node i
child(i, q) q-th child of node i
childrank(i) q = number of siblings to the left of node i
leafrank(i) number of leaves to the left and up to node i
leafselect(k) kth leaf of the tree
numleaves(i) number of leaves in the subtree of node i
leftmostleaf(i) / rightmostleaf(i) leftmost/rightmost leaf of node i"""

class BP(object):
    def __init__(self, B):
        assert B.sum() == (float(B.size) / 2)

        self.B = B

        self._k_index = [np.unique((~self.B).cumsum(), return_index=True)[1],
                         np.unique(self.B.cumsum(), return_index=True)[1] ]#- 1]
        self._r_index = [(~self.B).cumsum(), self.B.cumsum()]
        self._e_index = np.array([self._excess(i) for i in range(self.B.size)])
        self._closeopen_index = None

    def setcloseopen(self, closeopen):
        self._closeopen_index = closeopen

    def rank(self, t, i):
        """The number of occurrences of the bit t in B"""
        return self._r_index[t][i]

    def select(self, t, k):
        """The position in B of the kth occurrence of the bit t."""
        return self._k_index[t][k]

    def _excess(self, i):
        """Actually compute excess"""
        if i < 0:
            return 0  # wasn't stated as needed but appears so given testing
        return (2 * self.rank(1, i) - i) - 1

    def excess(self, i):
        """the number of opening minus closing parentheses in B[1, i]"""
        # same as: self.rank(1, i) - self.rank(0, i)
        return self._e_index[i]

    def fwdsearch(self, i, d):
        """Forward search for excess by depth"""
        b = self._e_index[i] + d

        for j in range(i + 1, len(self.B)):
            if self._e_index[j] == b:
                return j

        return -1  # wasn't stated as needed but appears so given testing

    def bwdsearch(self, i, d):
        """Backward search for excess by depth"""
        b = self._e_index[i] + d

        for j in range(i)[::-1]:
            if self._e_index[j] == b:
                return j

        return -1  # wasn't stated as needed but appears so given testing

    def close(self, i):
        """The position of the closing parenthesis that matches B[i]"""
        if not self.B[i]:
            # identity: the close of a closed parenthesis is itself
            return i

        if self._closeopen_index is not None:
            return self._closeopen_index[i]
        else:
            return self.fwdsearch(i, -1)

    def open(self, i):
        """The position of the opening parenthesis that matches B[i]"""
        if self.B[i] or i <= 0:
            # identity: the open of an open parenthesis is itself
            # the open of 0 is open. A negative index cannot be open, so just return
            return i
        ### if bwdsearch returns -1, should check and dump None?
        if self._closeopen_index is not None:
            return self._closeopen_index[i]
        else:
            return self.bwdsearch(i, 0) + 1

    def enclose(self, i):
        """The opening parenthesis of the smallest matching pair that contains position i"""
        if self.B[i]:
            return self.bwdsearch(i, -2) + 1
        else:
            return self.bwdsearch(i - 1, -2) + 1

    def rmq(self, i, j):
        """The leftmost minimum excess in i -> j"""
        return np.array([self.excess(k) for k in range(i, j + 1)]).argmin() + i

    def rMq(self, i, j):
        """The leftmost maximmum excess in i -> j"""
        return np.array([self.excess(k) for k in range(i, j + 1)]).argmax() + i

    def depth(self, i):
        """The depth of node i"""
        return self.excess(i)

    def root(self):
        """The root of the tree"""
        return 0

    def parent(self, i):
        """The parent of node i"""
        return self.enclose(i)

    def isleaf(self, i):
        """Whether the node is a leaf"""
        # publication describe this operation as "iff B[i+1] == 0" which is incorrect
        return self.B[i] and (not self.B[i + 1])

    def fchild(self, i):
        """The first child of i (i.e., the left child)

        fchild(i) = i + 1 (if i is not a leaf)"""
        if self.B[i]:
            if not self.isleaf(i):
                return i + 1
        else:
            return self.fchild(self.open(i))

    def lchild(self, i):
        """The last child of i (i.e., the right child)

        lchild(i) = open(close(i) − 1) (if i is not a leaf)"""
        pass

    def mincount(self, i, j):
        """number of occurrences of the minimum in excess(i), excess(i + 1), . . . , excess(j)."""
        excess, counts = np.unique([self.excess(k) for k in range(i, j + 1)], return_counts=True)
        return counts[excess.argmin()]

    def minselect(self, i, j, q):
        """position of the qth minimum in excess(i), excess(i + 1), . . . , excess(j)."""
        counts = np.array([self.excess(k) for k in range(i, j + 1)])
        index = counts == counts.min()

        if index.sum() < q:
            return None
        else:
            return i + index.nonzero()[0][q - 1]

    def nsibling(self, i):
        """The next sibling of i (i.e., the sibling to the right)

        nsibling(i) = close(i) + 1 (if the result j holds B[j] = 0 then i has no next sibling)"""
        if self.B[i]:
            pos = self.close(i) + 1
        else:
            pos = self.nsibling(self.open(i))

        if pos is None:
            return None
        elif pos >= len(self.B):
            return None
        elif self.B[pos]:
            return pos
        else:
            return None

    def psibling(self, i):
        """The previous sibling of i (i.e., the sibling to the left)

        psibling(i) = open(i − 1) (if B[i − 1] = 1 then i has no previous sibling)"""
        if self.B[i]:
            if self.B[max(0, i - 1)]:
                return None
            pos = self.open(i - 1)
        else:
            pos = self.psibling(self.open(i))

        if pos is None:
            return None
        elif pos < 0:
            return None
        elif self.B[pos]:
            return pos
        else:
            return None

    def preorder(self, i):
        """Preorder rank of node i"""
        # preorder(i) = rank1(i),
        if self.B[i]:
            return self.rank(1, i)
        else:
            return self.preorder(self.open(i))


    def preorderselect(self, k):
        """The node with preorder k"""
        # preorderselect(k) = select1(k),
        return self.select(1, k)

    def postorder(self, i):
        """Postorder rank of node i"""
        # postorder(i) = rank0(close(i)),
        if self.B[i]:
            return self.rank(0, self.close(i))
        else:
            ### i believe this is incorrect
            return self.rank(0, i)

    def postorderselect(self, k):
        """The node with postorder k"""
        # postorderselect(k) = open(select0(k)),
        return self.open(self.select(0, k))

    def isancestor(self, i, j):
        """Whether i is an ancestor of j"""
        # isancestor(i, j) iff i ≤ j < close(i)
        if i == j:
            return False

        if not self.B[i]:
            i = self.open(i)

        return i <= j < self.close(i)

    def subtree(self, i):
        """The number of nodes in the subtree of i"""
        # subtree(i) = (close(i) − i + 1)/2.
        if not self.B[i]:
            i = self.open(i)

        return (self.close(i) - i + 1) / 2

    def levelancestor(self, i, d):
        """ancestor j of i such that depth(j) = depth(i) − d"""
        # levelancestor(i, d) = bwdsearch(i, −d−1)+1
        if d <= 0:
            return -1

        if not self.B[i]:
            i = self.open(i)

        return self.bwdsearch(i, -d - 1) + 1

    def levelnext(self, i):
        """The next node with the same depth"""
        # levelnext(i) = fwdsearch(close(i), 1)
        return self.fwdsearch(self.close(i), 1)

    def levelprev(self, i):
        """The previous node with the same depth"""
        #levelprev(i) = open(bwdsearch(i, 0)+1)
        j = self.open(self.bwdsearch(self.open(i), 0))# + 1
        print(i, self.excess(i), self.excess(j), self.depth(i), self.depth(j))
        #print(i, self.bwdsearch(i, 0), self.open(self.bwdsearch(i, 0)))
        #return self.bwdsearch(self.open(i), 0)
        return j #self.bwdsearch(i - 1)
        #if not self.B[i]:
        #    i = self.open(i)

        #j = self.open(self.bwdsearch(i, 0))
        #if j < 0:
        #    return j
        #else:
        #    return j #+ 1

    def levelleftmost(self, d):
        #levelleftmost(d) = fwdsearch(0, d),
        pass

    def levelrightmost(self, d):
        #levelrightmost(d) = open(bwdsearch(2n + 1, d)).
        pass

    def degree(self, i):
        #degree(i) = mincount(i + 1, close(i) − 1),
        pass

    def child(i, q):
        # child(i, q) = minselect(i+1, close(i)−1, q−1)+1 for q > 1
        # (for q = 1 it is fchild(i)),
        pass

    def childrank(self, i):
        # childrank(i) = mincount(parent(i) + 1, i) + 1
        # unless B[i − 1] = 1
        # (in which case childrank(i) = 1)
        pass

    def lca(self, i, j):
        # lca(i, j) = parent(rmq(i, j) + 1)
        # unless isancestor(i, j)
        # (so lca(i, j) = i) or isancestor(j, i) (so lca(i, j) = j),
        if self.isancestor(i, j):
            return i
        elif self.isancestor(j, i):
            return j
        else:
            return self.parent(self.rmq(i, j) + 1)

    def deepestnode(self, i):
        # deepestnode(i) = rMq(i, close(i)),
        return self.rMq(self.open(i), self.close(i))

    def height(self, i):
        """the height of i (distance to its deepest node)"""
        # height(i) = excess(deepestnode(i)) − excess(i).
        return self.excess(self.deepestnode(i)) - self.excess(self.open(i))



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, BP development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------
from ._bp import BP
from ._io import parse_newick
from ._conv import to_skbio_treenode, from_skbio_treenode, to_skbio_treearray

__all__ = ['BP', 'parse_newick', 'to_skbio_treenode', 'from_skbio_treenode',
           'to_skbio_treearray']



from ._bp import BP

import numpy as np


def _set_node_metadata(ptr, token, names, lengths):
    """Inplace update of names and lengths given token details"""
    name = None
    length = 0.0

    if token.startswith(':'):
        length = float(token[1:])

    elif ':' in token:
        name, length = token.rsplit(':', 1)
        length = float(length)
        name = name.replace("'", "").replace('"', "")
    else:
        name = token.replace("'", "").replace('"', "")

    names[ptr] = name
    lengths[ptr] = length


def parse_newick(data):
    topology = _newick_to_bp(data)
    open_ = topology.open
    isleaf = topology.isleaf

    closeopen = np.zeros(len(topology.B), dtype=int)
    names = np.full(len(topology.B), None, dtype=object)
    lengths = np.zeros(len(topology.B), dtype=float)

    ptr = 0
    for token in _tokenizer(data):
        # it may be a performance gain if the tokenizer yielded:
        # (token_type, token). this would allow the elif check to be
        # elif token.type is not structural, resulting in 1 comparison vs
        # 5.

        if token == '(':
            # an open parenthesis never has metadata associated with it
            ptr += 1
            continue

        elif token not in '(),:;':
            open_ptr = open_(ptr)
            _set_node_metadata(open_ptr, token, names, lengths)
            if isleaf(ptr):
                # we have open, mapping close
                closeopen[ptr] = ptr + 1
                closeopen[ptr + 1] = ptr
                ptr += 2
            else:
                closeopen[ptr] = open_ptr
                closeopen[open_ptr] = ptr
                ptr += 1
    topology.setcloseopen(closeopen)
    return topology, names, lengths


def _newick_to_bp(data):
    """Convert newick to balanced parentheses

    Newick is _similar_ to BP, but differs notably at the tips of the tree.
    The complexity of the parse below comes from handling tips, and single
    descendents. Examples of situations that introduce this complexity are:

    ((a,b)) -> 11101000
    (a) -> 1100
    () -> 1100
    ((a,b),c) -> 1110100100
    (a,(b,c)) -> 1101101000

    Newick is not required to have node labels on tips, and the interpretation
    of a comma is dependent on prior state.

    The strategy undertaken is to reduce the newick string to only structural
    components. From there, the string is interpreted into tokens of: {"1",
    "0", "10", "100"}, which directly translate into the resulting balanced
    parentheses topology.

    It is very likely the case that this parser can be done better with
    improved efficiency.
    """
    topology = ''.join([c for c in data if c in '(),'])

    parentheses = np.full(len(topology), '', dtype='<U3')

    last = None
    single_descendent = False
    for idx, c in enumerate(topology):
        if c == '(':
            parentheses[idx] = '1'
            single_descendent = True
        elif c == ')':
            if single_descendent or last == ',':
                parentheses[idx] = '100'
                single_descendent = False
            else:
                parentheses[idx] = '0'
        else:
            if last != ')':
                parentheses[idx] = '10'
            single_descendent = False
        last = c

    return BP(np.array([c == '1' for c in ''.join(parentheses)], dtype=bool))


def _tokenizer(data):
    """Yield structural and non-structural slices from data"""
    last = 0
    for idx, i in enumerate(data):
        if i in '(,)':
            if idx - last > 0:
                yield data[last:idx]
            yield i
            last = idx + 1

    if data[-1] != ';':
        raise ValueError("Newick string is not semicolon terminated")

    if idx - last > 0:
        yield data[last:idx]

    yield data[-1]



# encoding: utf-8
# cython: profile=False, boundscheck=False, wraparound=False

from ._bp cimport BP
import time
import numpy as np
cimport numpy as np
cimport cython
np.import_array()


cdef void _set_node_metadata(np.uint32_t ptr, unicode token,
                             np.ndarray[object, ndim=1] names, 
                             np.ndarray[np.double_t, ndim=1] lengths):
    """Inplace update of names and lengths given token details"""
    cdef:
        np.double_t length
        Py_ssize_t split_idx, i
        unicode name

    name = None
    length = 0.0

    if token[0] == u':':
        length = np.double(token[1:])
    elif ':' in token:
        split_idx = token.rfind(':')
        name = token[:split_idx]
        length = np.double(token[split_idx + 1:])
        name = name.replace("'", "").replace('"', "")
    else:
        name = token.replace("'", "").replace('"', "")

    names[ptr] = name
    lengths[ptr] = length


cpdef parse_newick(unicode data):
    cdef:
        np.uint32_t ptr, open_ptr
        Py_ssize_t token_ptr, tmp, lag, datalen
        BP topology
        unicode token, last_token
        np.ndarray[object, ndim=1] names
        np.ndarray[np.double_t, ndim=1] lengths

    ##### probably can cache tip indices
    ##### probably can cache parent indices

    datalen = len(data)
    topology = _newick_to_bp(data)

    names = np.full(len(topology.B), None, dtype=object)
    lengths = np.zeros(len(topology.B), dtype=np.double)

    ptr = 0
    token_ptr = _ctoken(data, datalen, 0)
    token = data[0:token_ptr]
    last_token = None

    # lag reflects the scenario where ((x))y, where the label y gets may end
    # up being associated with an earlier unnamed vertex. lag represents the
    # offset between the topology pointer and the token pointer effectively.
    lag = 0
    while token != ';':
        if token == '(':
            # an open parenthesis never has metadata associated with it
            ptr += 1

        if (token == ')' or token == ',') and last_token == ')':
            # determine if there are unnamed/unlengthed nodes 
            lag += 1

        elif token not in '(),:;':
            ptr += lag
            lag = 0

            open_ptr = topology.open(ptr)
            _set_node_metadata(open_ptr, token, names, lengths)

            if topology.isleaf(ptr):
                ptr += 2
            else:
                ptr += 1

        last_token = token
        tmp = _ctoken(data, datalen, token_ptr)
        token = data[token_ptr:tmp]
        token_ptr = tmp

    topology.set_names(names)
    topology.set_lengths(lengths)
    return topology


cdef object _newick_to_bp(unicode data):
    """Convert newick to balanced parentheses

    Newick is _similar_ to BP, but differs notably at the tips of the tree.
    The complexity of the parse below comes from handling tips, and single
    descendents. Examples of situations that introduce this complexity are:

    ((a,b)) -> 11101000
    (a) -> 1100
    () -> 1100
    ((a,b),c) -> 1110100100
    (a,(b,c)) -> 1101101000

    Newick is not required to have node labels on tips, and the interpretation
    of a comma is dependent on prior state.

    The strategy undertaken is to reduce the newick string to only structural
    components. From there, the string is interpreted into tokens of: {"1",
    "0", "10", "100"}, which directly translate into the resulting balanced
    parentheses topology.

    It is very likely the case that this parser can be done better with
    improved efficiency.
    """
    cdef:
        Py_ssize_t i, topology_ptr, single_descendent
        Py_UCS4 c, last_c
        np.ndarray[np.uint8_t, ndim=1] topology
        
    potential_single_descendant = False

    topology = np.empty(len(data), dtype=np.uint8)
    topology_ptr = 0
    last_c = u'x'

    for i in range(len(data)):
        c = data[i]
        if c == u'(':
            # opening of a node
            topology[topology_ptr] = 1
            topology_ptr += 1
            last_c = c
            potential_single_descendant = True
        elif c == u')':
            # closing of a node
            if potential_single_descendant or last_c == u',':
                # we have a single descendant or a last child (i.e., ",)")
                topology[topology_ptr] = 1
                topology[topology_ptr + 1] = 0
                topology[topology_ptr + 2] = 0
                topology_ptr += 3
                potential_single_descendant = False
            else:
                # it is possible to still have a single descendant in the case
                # of a multiple single descendant: (...()...)
                topology[topology_ptr] = 0
                topology_ptr += 1
            last_c = c
        elif c == u',':
            if last_c != u')':
                # we have a new tip
                topology[topology_ptr] = 1
                topology[topology_ptr + 1] = 0
                topology_ptr += 2
            potential_single_descendant = False
            last_c = c
        else:
            # ignore non-structure
            pass

    return BP(topology[:topology_ptr])


cdef inline int _ccheck(Py_UCS4 c):
    """structure check"""
    cdef:
        Py_ssize_t i

    if c == u'(':
        return 1
    elif c == u')':
        return 1
    elif c == u',':
        return 1
    elif c == u';':
        return 1
    else:
        return 0


cdef inline int _is_quote(Py_UCS4 c):
    if c == u'"':
        return 1
    elif c == u"'":
        return 1
    else:
        return 0


cdef inline Py_ssize_t _ctoken(unicode data, Py_ssize_t datalen, Py_ssize_t start):
    cdef:
        Py_ssize_t idx, in_quote = 0
        Py_UCS4 c

    if start == (datalen - 1):
        return start + 1

    for idx in range(start, datalen):
        c = data[idx]

        if in_quote:
            if _is_quote(c):
                in_quote = 0
            continue
        else:
            if _is_quote(c):
                in_quote = 1
                continue

        if _ccheck(c):
            if idx == start:
                return idx + 1
            else:
                return idx
    
    return idx + 1



from unittest import TestCase, main
from bp import parse_newick


import numpy as np
import numpy.testing as npt


class NewickTests(TestCase):
    def test_parse_newick(self):
        in_ = "((a:2,b):1,(c:4,d)y:20,e)r;"

        exp_bp = [1, 1, 1, 0, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0]
        exp_n = ['r', None, 'a', None, 'b', None, None, 'y', 'c', None, 'd',
                 None, None, 'e', None, None]
        exp_l = [0, 1, 2, 0, 0, 0, 0, 20, 4, 0, 0, 0, 0, 0, 0, 0]

        obs_bp = parse_newick(in_)

        npt.assert_equal(obs_bp.B, np.asarray(exp_bp, dtype=bool))

        for i, (e_n, e_l) in enumerate(zip(exp_n, exp_l)):
            self.assertEqual(obs_bp.name(i), e_n)
            self.assertEqual(obs_bp.length(i), e_l)

    def test_parse_newick_complex(self):
        in_ = "(((a:1,b:2.5)c:6,d:8,(e),(f,g,(h:1,i:2)j:1)k:1.2)l,m:2)r;"

        exp_bp = [1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1,
                  1, 0, 1, 0, 0, 0, 0, 1, 0, 0]
        exp_n = ['r', 'l', 'c', 'a', None, 'b', None, None, 'd', None, None,
                 'e', None, None, 'k', 'f', None, 'g', None, 'j', 'h', None,
                 'i', None, None, None, None, 'm', None, None]
        exp_l = [0, 0, 6, 1, 0, 2.5, 0, 0, 8, 0, 0, 0, 0, 0, 1.2, 0, 0, 0, 0,
                 1, 1, 0, 2, 0, 0, 0, 0, 2, 0, 0]

        obs_bp = parse_newick(in_)

        npt.assert_equal(obs_bp.B, np.asarray(exp_bp, dtype=bool))

        for i, (e_n, e_l) in enumerate(zip(exp_n, exp_l)):
            self.assertEqual(obs_bp.name(i), e_n)
            self.assertEqual(obs_bp.length(i), e_l)

    def test_parse_newick_singledesc(self):
        in_ = "(((a)b)c,((d)e)f)r;"

        exp_bp = [1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0]
        exp_n = ['r', 'c', 'b', 'a', None, None, None, 'f', 'e', 'd', None,
                 None, None, None]

        obs_bp = parse_newick(in_)

        npt.assert_equal(obs_bp.B, np.asarray(exp_bp, dtype=bool))

        for i, e_n in enumerate(exp_n):
            self.assertEqual(obs_bp.name(i), e_n)

    def test_parse_newick_unnamed_singledesc(self):
        in_ = "((a,b)c,d,(e))r;"

        exp_bp = [1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0]
        exp_n = ['r', 'c', 'a', None, 'b', None, None, 'd', None, None, 'e',
                 None, None, None]

        obs_bp = parse_newick(in_)

        npt.assert_equal(obs_bp.B, np.asarray(exp_bp, dtype=bool))

        for i, e_n in enumerate(exp_n):
            self.assertEqual(obs_bp.name(i), e_n)

    def test_parse_newick_name_with_semicolon(self):
        in_ = "((a,(b,c):5)'d','e; foo':10,((f))g)r;"

        exp_bp = [1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0]
        exp_n = ['r', 'd', 'a', None, None, 'b', None, 'c', None, None, None,
                 'e; foo', None, 'g', None, 'f', None, None, None, None]
        exp_l = [0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 0]

        obs_bp = parse_newick(in_)

        npt.assert_equal(obs_bp.B, np.asarray(exp_bp, dtype=bool))

        for i, (e_n, e_l) in enumerate(zip(exp_n, exp_l)):
            self.assertEqual(obs_bp.name(i), e_n)
            self.assertEqual(obs_bp.length(i), e_l)


if __name__ == '__main__':
    main()



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, BP development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------
import os
from setuptools import setup
from setuptools.extension import Extension
from setuptools.command.build_py import build_py

import numpy as np

classes = """
    Development Status :: 4 - Beta
    License :: OSI Approved :: BSD License
    Topic :: Scientific/Engineering
    Programming Language :: Python
    Programming Language :: Python :: 3.5
    Operating System :: Unix
    Operating System :: POSIX
    Operating System :: MacOS :: MacOS X
"""
classifiers = [s.strip() for s in classes.split('\n') if s]

long_description = """An implementation of a balanced tree as described by
Cordova and Navarro"""

from Cython.Compiler.Options import directive_defaults

USE_CYTHON = os.environ.get('USE_CYTHON', True)
ext = '.pyx' if USE_CYTHON else '.c'
extensions = [Extension("bp._bp",
                        ["bp/_bp" + ext],
                        include_dirs=['BitArray/'],
                        library_dirs=['BitArray/'],
                        libraries=['bitarr']),
              Extension("bp._io",
                        ["bp/_io" + ext], ),
              Extension("bp._conv",
                        ["bp/_conv" + ext], ),
              Extension("bp._binary_tree",
                        ["bp/_binary_tree" + ext], ),
              Extension("bp.tests.test_bp_cy",
                        ["bp/tests/test_bp_cy" + ext],
                        include_dirs=['BitArray/'],
                        library_dirs=['BitArray/'],
                        libraries=['bitarr']),
              ]

extensions.extend([Extension("bp._ba",
                            ["bp/_ba" + ext],
                            include_dirs=['BitArray/'],
                             library_dirs=['BitArray/'],
                             libraries=['bitarr'])])





if USE_CYTHON:
    from Cython.Build import cythonize
    extensions = cythonize(extensions)


import subprocess
bitarr = os.path.join(os.path.abspath(__file__).rsplit('/', 1)[0], 'BitArray')

class BitArrayInstall(build_py):
    def run(self):
        subprocess.run(['make', '-C', bitarr, 'libbitarr.a'])
        build_py.run(self)


setup(name='iow',
      version="0.1.1",
      description='Balanced parentheses',
      author='Daniel McDonald',
      author_email='mcdonadt@colorado.edu',
      maintainer='Daniel McDonald',
      maintainer_email='mcdonadt@colorado.edu',
      url='https://github.com/wasade/improved-octo-waddle',
      packages=['bp'],
      ext_modules=extensions,
      include_dirs=[np.get_include(), 'BitArray/'],
      setup_requires=['numpy >= 1.9.2'],
      install_requires=[
          'numpy >= 1.9.2',
          'nose >= 1.3.7',
          'cython >= 0.24.1',
          'scikit-bio >= 0.5.0, < 0.6.0'],
      long_description=long_description,
      cmdclass={'build_py': BitArrayInstall})
